import 'dart:async';

import 'package:meta/meta.dart';

import 'core.dart';

class _EventStreamView<Event> extends StreamView<Event> {
  final StreamController<Event> _controller;

  _EventStreamView(StreamController<Event> controller)
      : _controller = controller,
        super(controller.stream);
}

class EventBus<Event> extends _EventStreamView<Event> implements EventBusBase<Event> {
  EventBus() : super(StreamController<Event>.broadcast());

  @override
  Stream<Event> get stream => this;

  @override
  @mustCallSuper
  void emit(Event event) {
    _controller.add(event);
  }

  @mustCallSuper
  Future addStream(Stream<Event> source, {bool? cancelOnError}) {
    return _controller.addStream(source, cancelOnError: cancelOnError);
  }

  @override
  @mustCallSuper
  StreamSubscription<T> on<T>(EventListener<T> listener) {
    Stream<T> result;
    if (T == dynamic) {
      result = this as Stream<T>;
    } else {
      result = where((event) => event is T).cast<T>();
    }
    return result.listen(listener);
  }

  Future get done => _controller.done;

  bool get hasListener => _controller.hasListener;

  @override
  bool get isClosed => _controller.isClosed;

  @override
  @mustCallSuper
  Future<void> close() async {
    await _controller.close();
  }
}
